# NM (normal mixed) EM-Algorithm (Algorithm 3.1)
# ==============================================================================
# ------------------------------------------------------------------------------
# Assuming that we have reasonable estimates for alpha_0, we want to compute
# the MLE of the log-concave signal

# Input: data_z = data (needs to be sorted)!!! z-values in R(eals) NOT p-values.
#        alp_hat = estimate for mixing proportion alpha_0
#        tol = determines when the recursion stops, difference between the weights 
#              must be smaller than tol for the algorithm to end
#        max_it = how many iterations should be executed at most
#        other arguments are for the recursion

# Output: f_1_z = last estimate for f_1
#         k = number of iterations

library(mixtools)
library(logcondens)

EM_normmixEM <- function(data_z, alp_hat, max_it = 250, k = 0, gamma_old = NULL,tol=5e-6, f_1_z = NULL, gamma_min = NULL, k_min = 0, res_min = NULL, min_difference = 1, prev_res = NULL) {
  
  n = length(data_z)
  
  # 1. Initialisation:
  if(k == 0) {
    # run an EM algorithm for a Gaussian mixture to get initial values of the signal using parametric normal fit.
    gamma_old <- rep(0, n)
    w <- 1-gamma_old
    A <- normalmixEM(data_z, lambda = c(1-alp_hat, alp_hat), k = 2,maxit=60, mean.constr = c(0,NA), sd.constr = c(1,NA)) 
    f_1_z <- dnorm(data_z, mean = A$mu[2], sd = A$sigma[2])
  }
  
  # 2. E-step: get new weights from posterior probability
  gamma_new <- (1-alp_hat)*dnorm(data_z)/((1-alp_hat)*dnorm(data_z) + (alp_hat)*f_1_z)
  
  # 3. M-step: compute the loc-concave estimates f_1_data_new with new weights
  w <- 1-gamma_new
  log_conc_est <- activeSetLogCon(data_z, w = w)
  if(log_conc_est$m == n) {
    f_1_z <- exp(log_conc_est$phi)
  } else {
    print("help");print(log_conc_est$m);return(list(f_1_z = f_1_z,k = k))
  }

  # 4. Replication:
  iterate_difference <- max(abs(gamma_new-gamma_old))
  if(iterate_difference < tol) {
    return(list(f_1_z = f_1_z,res=log_conc_est, k = k))
  } else if(k == max_it) {
    return(list(f_1_z = f_1_z,res=log_conc_est,k = k))
  } else {
    EM_normmixEM(data_z, alp_hat, k = k+1, gamma_old = gamma_new, f_1_z = f_1_z, tol = tol, max_it = max_it, gamma_min = gamma_min, k_min = k_min, res_min = res_min, min_difference = min_difference, prev_res = log_conc_est)
  }
}
# ------------------------------------------------------------------------------

# # ------------------------------------------------------------------------------
# # Example: with N(5,1) signal and mixing proportion alpha_0 = 0.4
# # ------------------------------------------------------------------------------
# 
# # Source Patra and Sen's code (modified for probit transofrmed data) to compute alpha_hat
# setwd("C:/newD/R-2.15.3/SemParMix")
# source("data_generation.R")
# source("patra_sen_Source/density_est.R")
# source("patra_sen_Source/dist_calc.R")
# source("patra_sen_Source/est_mix_model.R")
# 
# 
 gr0 = seq(-4,12,0.01)
 Plot.fun = function(size=500, alpha0=0.4){
 set.seed(1) 
 z1 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat1 <- est.mix.model(z1,method = "fixed")$alp.hat
 z1 <- sort(z1)
 A1 <- EM_normmixEM(z1,alp_hat1)
 f1hat1 = A1$f_1_z
 Max1 = max(dnorm(gr0,5,1),f1hat1) 
#############################
 set.seed(5) 
 z2 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat2 <- est.mix.model(z2,method = "fixed")$alp.hat
 z2 <- sort(z2)
 A2 <- EM_normmixEM(z2,alp_hat2)
 f1hat2 = A2$f_1_z
 Max2 = max(dnorm(gr0,5,1),f1hat2) 
####################################
set.seed(9) 
 z3 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat3 <- est.mix.model(z3,method = "fixed")$alp.hat
 z3 <- sort(z3)
 A3 <- EM_normmixEM(z3,alp_hat3)
 f1hat3 = A3$f_1_z
 Max3 = max(dnorm(gr0,5,1),f1hat3)
########################################## 
set.seed(13) 
 z4 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat4 <- est.mix.model(z4,method = "fixed")$alp.hat
 z4 <- sort(z4)
 A4 <- EM_normmixEM(z4,alp_hat4)
 f1hat4 = A4$f_1_z
 Max4 = max(dnorm(gr0,5,1),f1hat4)
 Max =  max(Max1,Max2,Max3,Max4)

 plot(gr0, dnorm(gr0,5,1), type = "l", xlab="", ylab="", ylim=c(0,Max), lwd=2.5)
 points(z1,f1hat1,col = "red", type="l", lty=2, lwd=3)
 points(z2,f1hat2,col = "blue", type="l",lty=3, lwd=3)
 points(z3,f1hat3,col = "magenta", type="l",lty=4, lwd=3)
 points(z4,f1hat3,col = "orange", type="l",lty=6, lwd=3)


}
Plot.fun(size=500,alpha0=0.4)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.4 and n=500") 

Plot.fun(size=5000,alpha0=0.4)
title("The log-concave MLE with signal N(5,1), alpha0 = 0.4 and n=5000") 



Plot.fun(size=500,alpha0=0.8)
title("The log-concave MLE with signal N(5,1), alpha0 = 0.8 and n=500") 

Plot.fun(size=5000,alpha0=0.8)
title("The log-concave MLE with signal N(5,1), alpha0 = 0.8 and n=5000") 



### # ------------------------------------------------------------------------------
# # Example: with N(5,1) signal and mixing proportions alpha_0 = 0.1 and 0.05
# # ------------------------------------------------------------------------------
# 
# # Source Patra and Sen's code (modified for probit transofrmed data) to compute alpha_hat
# setwd("C:/newD/R-2.15.3/SemParMix")
# source("data_generation.R")
# source("patra_sen_Source/density_est.R")
# source("patra_sen_Source/dist_calc.R")
# source("patra_sen_Source/est_mix_model.R")
# 
# 
 gr0 = seq(-4,12,0.01)
 Plot.fun = function(size=500, alpha0=0.1){
 set.seed(1) 
 z1 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat1 <- est.mix.model(z1,method = "fixed")$alp.hat
 z1 <- sort(z1)
 A1 <- EM_normmixEM(z1,alp_hat1)
 f1hat1 = A1$f_1_z
 Max1 = max(dnorm(gr0,5,1),f1hat1) 
#############################
 set.seed(5) 
 z2 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat2 <- est.mix.model(z2,method = "fixed")$alp.hat
 z2 <- sort(z2)
 A2 <- EM_normmixEM(z2,alp_hat2)
 f1hat2 = A2$f_1_z
 Max2 = max(dnorm(gr0,5,1),f1hat2) 
####################################
set.seed(9) 
 z3 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat3 <- est.mix.model(z3,method = "fixed")$alp.hat
 z3 <- sort(z3)
 A3 <- EM_normmixEM(z3,alp_hat3)
 f1hat3 = A3$f_1_z
 Max3 = max(dnorm(gr0,5,1),f1hat3)
########################################## 
set.seed(13) 
 z4 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rnorm(n,5,1))}))
 alp_hat4 <- est.mix.model(z4,method = "fixed")$alp.hat
 z4 <- sort(z4)
 A4 <- EM_normmixEM(z4,alp_hat4)
 f1hat4 = A4$f_1_z
 Max4 = max(dnorm(gr0,5,1),f1hat4)
 Max =  max(Max1,Max2,Max3,Max4)
print((alp_hat1 +  alp_hat2 + alp_hat3 + alp_hat4)/4) 
 plot(gr0, dnorm(gr0,5,1), type = "l", xlab="", ylab="", ylim=c(0,Max), lwd=2.5)
 points(z1,f1hat1,col = "red", type="l", lty=2, lwd=3)
 points(z2,f1hat2,col = "blue", type="l",lty=3, lwd=3)
 points(z3,f1hat3,col = "magenta", type="l",lty=4, lwd=3)
 points(z4,f1hat4,col = "orange", type="l",lty=6, lwd=3)


}

Plot.fun(size=500,alpha0=0.4)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.4 and n=500") 

###mean alpha_n = 0.397

Plot.fun(size=5000,alpha0=0.4)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.4 and n=5000") 

###mean alpha_n = 0.396

Plot.fun(size=500,alpha0=0.8)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.8 and n=500") 

###mean alpha_n = 0.768

Plot.fun(size=5000,alpha0=0.8)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.8 and n=5000") 

###mean alpha_n =0.781



Plot.fun(size=500,alpha0=0.1)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.1 and n=500") 

###mean alpha_n = 0.13

Plot.fun(size=5000,alpha0=0.1)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.1 and n=5000") 

###mean alpha_n = 0.11


Plot.fun(size=5000,alpha0=0.05)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.05 and n=5000") 
###mean alpha_n = 0.060



Plot.fun(size=10000,alpha0=0.05)

title("The log-concave MLE with signal N(5,1), alpha0 = 0.05 and n=10000") 





#######################################################################################
gr1 = seq(-4,8,0.01)
 PlotGamma.fun = function(size=500, alpha0=0.4){
 set.seed(100) 
 z1 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rgamma(n,shape=4,rate=2))}))
 alp_hat1 <- est.mix.model(z1,method = "fixed")$alp.hat
 z1 <- sort(z1)
 A1 <- EM_normmixEM(z1,alp_hat1)
 f1hat1 = A1$f_1_z
 Max1 = max(dgamma(gr1,4,2),f1hat1) 
#############################
 set.seed(200) 
 z2 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rgamma(n,shape=4,rate=2))}))
 alp_hat2 <- est.mix.model(z2,method = "fixed")$alp.hat
 z2 <- sort(z2)
 A2 <- EM_normmixEM(z2,alp_hat2)
 f1hat2 = A2$f_1_z
 Max2 = max(dgamma(gr1,4,2),f1hat2) 
####################################
set.seed(300) 
 z3 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rgamma(n,shape=4,rate=2))}))
 alp_hat3 <- est.mix.model(z3,method = "fixed")$alp.hat
 z3 <- sort(z3)
 A3 <- EM_normmixEM(z3,alp_hat3)
 f1hat3 = A3$f_1_z
 Max3 = max(dgamma(gr1,4,2),f1hat3)
########################################## 
set.seed(400) 
 z4 <- (data.gen(n=size, alpha = alpha0, alt_fct = function(n){return(rgamma(n,shape=4,rate=2))}))
 alp_hat4 <- est.mix.model(z4,method = "fixed")$alp.hat
 z4 <- sort(z4)
 A4 <- EM_normmixEM(z4,alp_hat4)
 f1hat4 = A4$f_1_z
 Max4 = max(dgamma(gr1,4,2),f1hat4)
 Max =  max(Max1,Max2,Max3,Max4)
 print((alp_hat1 + alp_hat2 + alp_hat3 + alp_hat4)/4)
 plot(gr1, dgamma(gr1,shape=4,rate=2), type = "l", xlab="", ylab="", ylim=c(0,0.5), lwd=2.5)
 points(z1,f1hat1,col = "red", type="l", lty=2, lwd=3)
 points(z2,f1hat2,col = "blue", type="l",lty=3, lwd=3)
 points(z3,f1hat3,col = "magenta", type="l",lty=4, lwd=3)
 points(z4,f1hat4,col = "orange", type="l",lty=6, lwd=3)


}

PlotGamma.fun(size=500,alpha0=0.8)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.8 and n=500") 

#mean alpha_n = 0.743
 
PlotGamma.fun(size=5000,alpha0=0.8)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.8 and n=5000") 

#mean alpha_n= 0.777


PlotGamma.fun(size=500,alpha0=0.4)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.4 and n=500") 

#mean alpha_n =0.388

PlotGamma.fun(size=5000,alpha0=0.4)
title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.4 and n=5000") 
#mean alpha_n =  0.396


PlotGamma.fun(size=500,alpha0=0.1)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.1 and n=500") 

#mean alpha_n = 0.098

PlotGamma.fun(size=5000,alpha0=0.1)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.1 and n=5000") 

#mean alpha_n =0.112



PlotGamma.fun(size=5000,alpha0=0.05)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.05 and n=5000") 

#mean alpha_n =0.064


PlotGamma.fun(size=10000,alpha0=0.05)

title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.05 and n=10000") 

#mean alpha_n = 



PlotGamma.fun(size=500,alpha0=0.8)
title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.8 and n=500") 

PlotGamma.fun(size=5000,alpha0=0.8)
title("The log-concave MLE with signal Gamma(4,2), alpha0 = 0.8 and n=5000") 


###########################################################################################

EM_normmixEM_test <- function(x0, data_z, alp_hat, max_it = 250, k = 0, gamma_old = NULL,tol=5e-6, f_1_z = NULL, gamma_min = NULL, k_min = 0, res_min = NULL, min_difference = 1, prev_res = NULL) {
  
  n = length(data_z)
  
  # 1. Initialisation:
  if(k == 0) {
    # run an EM algorithm for a Gaussian mixture to get initial values of the signal using parametric normal fit.
    gamma_old <- rep(0, n)
    w <- 1-gamma_old
    A <- normalmixEM(data_z, lambda = c(1-alp_hat, alp_hat), k = 2, mean.constr = c(0,NA), sd.constr = c(1,NA)) 
    f_1_z <- dnorm(data_z, mean = A$mu[2], sd = A$sigma[2])
  }
  
  # 2. E-step: get new weights from posterior probability
  gamma_new <- (1-alp_hat)*dnorm(data_z)/((1-alp_hat)*dnorm(data_z) + (alp_hat)*f_1_z)
  
  # 3. M-step: compute the loc-concave estimates f_1_data_new with new weights
  w <- 1-gamma_new
  log_conc_est <- activeSetLogCon(data_z, w = w)
  if(log_conc_est$m == n) {
    f_1_z <- exp(log_conc_est$phi) 
    print("Are you going to print something?")
 
    fhat.x0 <- evaluateLogConDens(xs=x0, res=log_conc_est, which = 2)[3]
  } else {
    print("help");print(log_conc_est$m);return(list(f_1_z=f_1_z, fhat.x0 = fhat.x0,k = k))
  }

  # 4. Replication:
  iterate_difference <- max(abs(gamma_new-gamma_old))
  if(iterate_difference < tol) {
    return(list(f_1_z=f_1_z, fhat.x0 = fhat.x0,k = k))
  } else if(k == max_it) {
    return(list(f_1_z=f_1_z, fhat.x0 = fhat.x0,k = k))
  } else {
    EM_normmixEM(data_z, alp_hat, k = k+1, gamma_old = gamma_new, f_1_z=f_1_z, tol = tol, max_it = max_it, gamma_min = gamma_min, k_min = k_min, res_min = res_min, min_difference = min_difference, prev_res = log_conc_est)
  }
}




############################################################################################
scaledernorm = function(M=500, Ta=5000,x0=0,mu=5, alpha0=0.4){
  
 f0.x0  =  dnorm(x=x0, mean=mu)
 out = NULL
 for(j in 1:M){
 print(j)
 z1.j <- (data.gen(n=Ta, alpha = alpha0, alt_fct = function(n){return(rnorm(n,mean=mu,1))}))
 alp_hat1.j <- est.mix.model(z1.j,method = "fixed")$alp.hat
 z1.j <- sort(z1.j)
 Res.j <- EM_normmixEM(z1.j,alp_hat1.j)$res
 fhatx0.j = evaluateLogConDens(xs=x0, Res.j, which = 2)[3]

 
out =  c(out, Ta^{2/5}*(fhatx0.j - f0.x0))

}

out

}


# # ------------------------------------------------------------------------------


ErrorNormmu5x1.5000 = scaledernorm(M=500, Ta=5000, x0=1, mu=5, alpha0=0.6)

ErrorNormmu5x1 = scaledernorm(M=100, size=10000, x0=1, mu=5, alpha0=0.6)





ErrorNormmu5x3.5000 = scaledernorm(M=500, Ta=5000, x0=3, mu=5, alpha0=0.6)



ErrorNormmu5x5 = scaledernorm(M=500, Ta=5000, x0=5, mu=5, alpha0=0.6)
ErrorNormmu5x3.0.4 = scaledernorm(M=500, Ta=5000, x0=3, mu=5, alpha0=0.4)
ErrorNormmu5x5.0.4 = scaledernorm(M=500, Ta=5000, x0=5, mu=5, alpha0=0.4)

hist(ErrorNormmu5x3.5000, prob=TRUE,nclass=20, xlab="", ylab="", main="Histogram of the scaled error, N(5,1), x_0=3, alpha_0=0.6")
points(density(ErrorNormmu5x3.5000), type="l", lwd=2.5)
points(sort(ErrorNormmu5x3.5000), dnorm(sort(ErrorNormmu5x3.5000), mean(ErrorNormmu5x3.5000), sd(ErrorNormmu5x3.5000)), type="l",lty=2, lwd=2.5)


hist(ErrorNormmu5x5, prob=TRUE,nclass=20, xlab="", ylab="", main="Histogram of the scaled error, N(5,1), x_0=5, alpha_0=0.6")
points(density(ErrorNormmu5x5), type="l", lwd=2.5)
points(sort(ErrorNormmu5x5), dnorm(sort(ErrorNormmu5x5), mean(ErrorNormmu5x5), sd(ErrorNormmu5x5)), type="l",lty=2, lwd=2.5)


#########################################################
hist(ErrorNormmu5x3.0.4, prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, N(5,1), x_0=3, alpha_0=0.4")
points(density(ErrorNormmu5x3.0.4), type="l", lwd=2.5)
points(sort(ErrorNormmu5x3.0.4), dnorm(sort(ErrorNormmu5x3.0.4), mean(ErrorNormmu5x3.0.4), sd(ErrorNormmu5x3.0.4)), type="l",lty=2, lwd=2.5)



hist(ErrorNormmu5x5.0.4, prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, N(5,1), x_0=5, alpha_0=0.4")
points(density(ErrorNormmu5x5.0.4), type="l", lwd=2.5)
points(sort(ErrorNormmu5x5.0.4), dnorm(sort(ErrorNormmu5x5.0.4), mean(ErrorNormmu5x5.0.4), sd(ErrorNormmu5x5.0.4)), type="l",lty=2, lwd=2.5)





################################################################


scaledergamma = function(M=500, Ta=5000,x0=1, alpha0=0.4){
  
 f0.x0  =  dgamma(x=x0, shape=4, rate=2)
 out = NULL
 for(j in 1:M){
 print(j)
 z1.j <- (data.gen(n=Ta, alpha = alpha0, alt_fct = function(n){return(rgamma(n,shape=4,rate=2))}))
 alp_hat1.j <- est.mix.model(z1.j,method = "fixed")$alp.hat
 z1.j <- sort(z1.j)
 Res.j <- EM_normmixEM(z1.j,alp_hat1.j)$res
 fhatx0.j = evaluateLogConDens(xs=x0, Res.j, which = 2)[3]

 
out =  c(out, Ta^{2/5}*(fhatx0.j - f0.x0))

}

out

}


hist(ErrorGammax0.5.5000, prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=0.5, alpha_0=0.6")
points(density(ErrorGammax0.5.5000), type="l", lwd=2.5)
points(sort(ErrorGammax0.5.5000), dnorm(sort(ErrorGammax0.5.5000), mean(ErrorGammax0.5.5000), sd(ErrorGammax0.5.5000)), type="l", lty=2, lwd=2.5)

hist(ErrorGammax1.5, prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=1.5, alpha_0=0.6")
points(density(ErrorGammax1.5), type="l", lwd=2.5)
points(sort(ErrorGammax1.5), dnorm(sort(ErrorGammax1.5), mean(ErrorGammax1.5), sd(ErrorGammax1.5)), type="l", lty=2, lwd=2.5)



hist(ErrorGammax1.5, prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=1.5, alpha_0=0.6")
points(density(ErrorGammax1.5), type="l", lwd=2.5)
points(sort(ErrorGammax1.5), dnorm(sort(ErrorGammax1.5), mean(ErrorGammax1.5), sd(ErrorGammax1.5)), type="l", lty=2, lwd=2.5)


ErrorGammax0.5 = scaledergamma(M=500, Ta=5000, x0=0.5, alpha0=0.6)

ErrorGammax1.5 = scaledergamma(M=500, Ta=5000, x0=1.5, alpha0=0.6)


ErrorGammax0.5.0.4 = scaledergamma(M=500, Ta=5000, x0=0.5, alpha0=0.4)

ErrorGammax1.5.0.4 = scaledergamma(M=500, Ta=5000, x0=1.5, alpha0=0.4)

hist(ErrorGammax0.5,prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=0.5, alpha_0=0.6")
points(density(ErrorGammax0.5), type="l", lwd=2.5)
points(sort(ErrorGammax0.5), dnorm(sort(ErrorGammax0.5), mean(ErrorGammax0.5), sd(ErrorGammax0.5)), type="l", lty=2, lwd=2.5)



hist(ErrorGammax1.5,prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=1.5, alpha_0=0.6")
points(density(ErrorGammax1.5), type="l", lwd=2.5)
points(sort(ErrorGammax1.5), dnorm(sort(ErrorGammax1.5), mean(ErrorGammax1.5), sd(ErrorGammax0.5)), type="l", lty=2, lwd=2.5)




hist(ErrorGammax0.5.0.4,prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=0.5, alpha_0=0.4")
points(density(ErrorGammax0.5.0.4), type="l", lwd=2.5)
points(sort(ErrorGammax0.5.0.4), dnorm(sort(ErrorGammax0.5.0.4), mean(ErrorGammax0.5.0.4), sd(ErrorGammax0.5.0.4)), type="l", lty=2, lwd=2.5)


hist(ErrorGammax1.5.0.4,prob=TRUE,nclass=25, xlab="", ylab="", main="Histogram of the scaled error, Gamma(4,2), x_0=1.5, alpha_0=0.4")
points(density(ErrorGammax1.5.0.4), type="l", lwd=2.5)
points(sort(ErrorGammax1.5.0.4), dnorm(sort(ErrorGammax1.5.0.4), mean(ErrorGammax1.5.0.4), sd(ErrorGammax1.5.0.4)), type="l", lty=2, lwd=2.5)




z1.test <- (data.gen(n=1000, alpha = 0.4, alt_fct = function(n){return(rnorm(n,mean=5,1))}))
 alp_hat1 <- est.mix.model(z1.test,method = "fixed")$alp.hat
 z1.test <- sort(z1.test)
 A1.test <- EM_normmixEM_test(z1.test,alp_hat1)
 A1.test <- EM_normmixEM(z1.test,alp_hat1)

Test= activeSetLogCon(z1.test, w = rep(1,1000))
evaluateLogConDens(xs=0, Res.test, which = 2)[3]


